import os
import re
from calendar import timegm
from email.utils import formatdate, parsedate
from mock import Mock
from tempfile import mkstemp
from fresco import FrescoApp
from fresco.static import serve_static_file


class TestServeStaticFile():

    def setup(self):

        fh, tmpname = mkstemp()
        self.tmpname = tmpname
        self.mtime = int(os.path.getmtime(self.tmpname))
        os.close(fh)

    def teardown(self):
        os.unlink(self.tmpname)

    def parse_lmh(self, last_modified):
        return timegm(parsedate(last_modified))

    def make_ims(self, since, tz=0):
        return {'HTTP_IF_MODIFIED_SINCE': formatdate(since, tz)}

    def test_sets_last_modified_header(self):
        with FrescoApp().requestcontext():
            response = serve_static_file(self.tmpname)
            response.content.close()

        last_modified = response.get_header('Last-Modified')
        assert self.parse_lmh(last_modified) == self.mtime

    def test_full_response_when_if_modified_before_mtime(self):
        with FrescoApp().requestcontext(**self.make_ims(0)):
            response = serve_static_file(self.tmpname)
            response.content.close()

        last_modified = response.get_header('Last-Modified')
        assert response.status_code == 200
        assert self.parse_lmh(last_modified) == self.mtime

    def test_304_when_if_modified_after_mtime(self):
        with FrescoApp().requestcontext(**self.make_ims(self.mtime + 1)):
            response = serve_static_file(self.tmpname)
        assert response.status_code == 304

    def test_304_when_if_modified_after_mtime_with_tz1(self):
        with FrescoApp().requestcontext(**self.make_ims(self.mtime + 1, +1)):
            response = serve_static_file(self.tmpname)
        assert response.status_code == 304

    def test_304_when_if_modified_after_mtime_with_tz2(self):
        with FrescoApp().requestcontext(**self.make_ims(self.mtime + 1, -1)):
            response = serve_static_file(self.tmpname)
        assert response.status_code == 304

    def test_last_modified_format_is_correct(self):
        with FrescoApp().requestcontext():
            response = serve_static_file(self.tmpname)
            response.content.close()

        assert re.match(r'^\w{3}, \d{1,2} \w{3} \d{4} \d\d:\d\d:\d\d GMT',
                        response.get_header('last-modified'))

    def test_filewrapper_used_if_present(self):
        with FrescoApp().requestcontext() as c:
            file_wrapper = Mock(return_value=[])
            c.request.environ['wsgi.file_wrapper'] = file_wrapper
            serve_static_file(self.tmpname)

        file_wrapper.assert_called_once()



from mock import Mock

from fresco import Route, GET, FrescoApp
from fresco import routeargs


class TestRouteKwargs(object):

    def test_routekwarg_configured(self):

        A = routeargs.RouteKwarg()
        route = Route('/', GET, lambda r: None, x=A)
        assert A.route is route
        assert A.name is 'x'

    def test_routekwarg_value_passed(self):

        view = Mock()
        routekwarg = Mock(spec=routeargs.RouteKwarg)
        routekwarg.return_value = 'xyzzy'

        app = FrescoApp()
        app.route('/', GET, view, x=routekwarg)
        with app.requestcontext('/') as c:
            app.view()
            routekwarg.assert_called_with(c.request)
            view.assert_called_with(x='xyzzy')

    def test_queryarg_value_passed(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.QueryArg())
        with app.requestcontext('/?x=foo'):
            app.view()
            view.assert_called_with(x='foo')

    def test_formarg_value_passed(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.FormArg())
        with app.requestcontext('/?x=foo'):
            app.view()
            view.assert_called_with(x='foo')

    def test_sessionarg_value_passed(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.SessionArg())
        with app.requestcontext('/?x=foo') as c:
            c.request.environ[c.request.SESSION_ENV_KEY] = {'x': 'foo'}
            app.view()
            view.assert_called_with(x='foo')

    def test_cookiearg_value_passed(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.CookieArg())
        with app.requestcontext('/', HTTP_COOKIE='x=foo'):
            app.view()
            view.assert_called_with(x='foo')

    def test_cookiearg_listvalue_passed(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.CookieArg([str]))
        with app.requestcontext('/', HTTP_COOKIE='x=foo;x=bar'):
            app.view()
            view.assert_called_with(x=['foo', 'bar'])

    def test_requestarg_value_converted(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.FormArg(float))
        with app.requestcontext('/?x=0'):
            app.view()
            view.assert_called_with(x=0.0)

    def test_requestarg_default_value(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.FormArg(default='d'))
        with app.requestcontext('/'):
            app.view()
            view.assert_called_with(x='d')

    def test_requestarg_returns_badrequest(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.FormArg())
        with app.requestcontext('/'):
            assert 'Bad Request' in app.view().status


class TestRequestObject(object):

    def test_it_passes_the_request(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.RequestObject())
        with app.requestcontext('/') as c:
            app.view()
            x = view.call_args[1]['x']
            assert x is c.request


class TestFormData(object):

    def test_it_passes_the_form_dict(self):
        view = Mock()
        app = FrescoApp()
        app.route('/', GET, view, x=routeargs.FormData())
        with app.requestcontext('/') as c:
            app.view()
            print(view.call_args)
            x = view.call_args[1]['x']
            assert x is c.request.form



# coding=UTF-8

from io import BytesIO
from nose.tools import assert_equal

from fresco import FrescoApp
from fresco.compat import PY3, quote

context = FrescoApp().requestcontext


class TestRequestProperties(object):

    def test_url_script_name_only(self):
        with context(SCRIPT_NAME='/foo/bar', PATH_INFO='') as c:
            assert_equal(c.request.url, "http://localhost/foo/bar")

    def test_url_script_name_path_info(self):
        with context(SCRIPT_NAME='/foo/bar', PATH_INFO='/baz') as c:
            assert_equal(c.request.url, "http://localhost/foo/bar/baz")

    def test_url_normalizes_host_port(self):
        with context(HTTP_HOST='localhost:80') as c:
            assert_equal(c.request.url, "http://localhost/")
        with context(HTTP_HOST='localhost:81') as c:
            assert_equal(c.request.url, "http://localhost:81/")

    def test_url_normalizes_host_ssl_port(self):
        with context(environ={'wsgi.url_scheme': 'https'},
                     HTTP_HOST='localhost:443') as c:
            assert_equal(c.request.url, "https://localhost/")

    def test_url_ignores_server_port_if_host_header_present(self):
        with context(environ={'wsgi.url_scheme': 'https', 'SERVER_PORT': '81'},
                     HTTP_HOST='localhost') as c:
            assert_equal(c.request.url, "https://localhost/")

    def test_as_string(self):
        with context('http://example.org/foo?bar=baz') as c:
            assert_equal(str(c.request),
                         '<Request GET http://example.org/foo?bar=baz>')

    def test_url_returns_full_url(self):
        with context('http://example.org/foo?bar=baz') as c:
            assert_equal(c.request.url, 'http://example.org/foo?bar=baz')

    def test_path_returns_correct_path_when_script_name_empty(self):
        with context(SCRIPT_NAME='', PATH_INFO='/foo/bar') as c:
            assert_equal(c.request.path, '/foo/bar')

    def test_path_returns_correct_path(self):
        with context(SCRIPT_NAME='/foo', PATH_INFO='/bar') as c:
            assert_equal(c.request.path, '/foo/bar')

    def test_query_decodes_unicode(self):
        with context('/?q=%C3%A0') as c:
            assert_equal(c.request.query['q'], b'\xc3\xa0'.decode('utf8'))

    def test_form_decodes_unicode(self):
        with context('/?q=%C3%A0') as c:
            assert_equal(c.request.form['q'], b'\xc3\xa0'.decode('utf8'))


class TestPathEncoding(object):

    if PY3:
        unquoted_url = '/el niño'
        quoted_url = quote(unquoted_url)
        wsgi_url = unquoted_url.encode('utf8').decode('latin1')
    else:
        unquoted_url = unicode('/el niño', 'UTF-8')
        quoted_url = quote(unquoted_url.encode('UTF-8'))
        wsgi_url = unquoted_url.encode('UTF-8')

    def test_SCRIPT_NAME_PATH_INFO_latin1_decoded(self):

        with context(SCRIPT_NAME=self.unquoted_url,
                     PATH_INFO=self.unquoted_url) as c:
            assert c.request.environ['SCRIPT_NAME'] == self.wsgi_url
            assert c.request.environ['PATH_INFO'] == self.wsgi_url

    def test_url_is_quoted(self):

        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
            assert c.request.url == 'http://localhost' + self.quoted_url

    def test_application_url_is_quoted(self):

        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
            assert c.request.application_url == 'http://localhost' + \
                                                    self.quoted_url

    def test_parsed_url_is_quoted(self):
        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
            assert c.request.parsed_url.path == self.quoted_url

    def test_path_is_unquoted(self):
        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
            assert c.request.path == self.unquoted_url

    def test_script_name_is_unquoted(self):
        with context(SCRIPT_NAME=self.unquoted_url, PATH_INFO='') as c:
            assert c.request.script_name == self.unquoted_url

    def test_path_info_is_unquoted(self):
        with context(SCRIPT_NAME='', PATH_INFO=self.unquoted_url) as c:
            assert c.request.path_info == self.unquoted_url


class TestMakeURL(object):

    def test_it_returns_request_url(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.make_url(),
                         'http://localhost/script/pathinfo')

    def test_it_doesnt_double_quote_request_url(self):
        with context(SCRIPT_NAME='/script name', PATH_INFO='/path info') as c:
            assert_equal(c.request.make_url(),
                         'http://localhost/script%20name/path%20info')

    def test_it_doesnt_double_quote_supplied_path_info(self):
        with context() as c:
            assert_equal(c.request.make_url(PATH_INFO='/x y'),
                         'http://localhost/x%20y')

    def test_can_replace_path(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.make_url(path='/foo bar'),
                         'http://localhost/foo%20bar')

    def test_it_joins_path(self):
        with context(SCRIPT_NAME='/script name', PATH_INFO='/path info/') as c:
            assert_equal(c.request.make_url(path='a/b c'),
                         'http://localhost/script%20name/path%20info/a/b%20c')

    def test_query_not_included_by_default(self):
        with context(QUERY_STRING='query=foo') as c:
            assert_equal(c.request.make_url(),
                         'http://localhost/')

    def test_query_dict(self):
        with context() as c:
            assert_equal(c.request.make_url(query={'a': 1, 'b': '2 3'}),
                         'http://localhost/?a=1;b=2+3')

    def test_query_kwargs(self):
        with context() as c:
            assert_equal(c.request.make_url(query={'a': 1}, b=2),
                         'http://localhost/?a=1;b=2')

    def test_unicode_path(self):
        e = b'\xc3\xa9'.decode('utf8')  # e-acute
        with context() as c:
            assert_equal(c.request.make_url(path=e),
                         'http://localhost/%C3%A9')

    def test_unicode_path_info(self):
        e = b'\xc3\xa9'.decode('utf8')  # e-acute
        with context() as c:
            assert_equal(c.request.make_url(PATH_INFO=e),
                         'http://localhost/%C3%A9')

    def test_unicode_query(self):
        e = b'\xc3\xa9'.decode('utf8')  # e-acute
        with context() as c:
            assert_equal(c.request.make_url(query={'e': e}),
                         'http://localhost/?e=%C3%A9')

    def test_it_quotes_paths(self):
        with context() as c:
            assert_equal(c.request.make_url(path='/a b'),
                         'http://localhost/a%20b')


class TestResolveURL(object):

    def test_relative_path(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.resolve_url('foo'),
                         'http://localhost/script/foo')

    def test_absolute_path_unspecified_relative(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.resolve_url('/foo', relative='app'),
                         'http://localhost/script/foo')

    def test_absolute_path_app_relative(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.resolve_url('/foo', relative='app'),
                         'http://localhost/script/foo')

    def test_absolute_path_server_relative(self):
        with context(SCRIPT_NAME='/script', PATH_INFO='/pathinfo') as c:
            assert_equal(c.request.resolve_url('/foo', relative='server'),
                         'http://localhost/foo')

    def test_ignores_server_port_if_host_header_present(self):
        with context(environ={'wsgi.url_scheme': 'https', 'SERVER_PORT': '81'},
                     HTTP_HOST='localhost') as c:
            assert_equal(c.request.resolve_url('/foo'),
                         'https://localhost/foo')


class TestCurrentRequest(object):

    def test_currentrequest_returns_current_request(self):
        from fresco import currentrequest
        with context() as c:
            assert currentrequest() is c.request

    def test_currentrequest_returns_None(self):
        from fresco import currentrequest
        assert currentrequest() is None


class TestFiles(object):

    filename = 'test.txt'
    filedata = '123456\n'
    boundary = '---------------------------1234'

    post_data = (
        '--{boundary}\r\n'
        'Content-Disposition: form-data; '
            'name="uploaded_file"; filename="{filename}"\r\n'
        'Content-Type: text/plain\r\n'
        '\r\n'
        '{filedata}\r\n'
        '--{boundary}'
        '--\r\n'
    ).format(boundary=boundary, filedata=filedata, filename=filename)\
     .encode('latin1')

    request_args = {'wsgi_input': post_data,
                    'REQUEST_METHOD': 'POST',
                    'CONTENT_TYPE': 'multipart/form-data; boundary=' + \
                                        boundary,
                    'CONTENT_LENGTH': str(len(post_data))}

    def test_files_populated(self):

        with FrescoApp().requestcontext(**self.request_args) as c:
            request = c.request

        assert len(request.files) == 1
        assert 'uploaded_file' in request.files

    def test_file_content_available(self):

        with FrescoApp().requestcontext(**self.request_args) as c:
            request = c.request

        b = BytesIO()
        request.files['uploaded_file'].save(b)
        assert b.getvalue() == self.filedata.encode('latin1')

    def test_headers_available(self):

        with FrescoApp().requestcontext(**self.request_args) as c:
            request = c.request

        assert request.files['uploaded_file'].headers['content-type'] \
                == 'text/plain'
        assert request.files['uploaded_file'].filename == self.filename



from nose.tools import assert_equal, assert_raises

from fresco.util import security

class TestSecurity(object):

    def test_check_equal_constant_time_returns_equality(self):
        assert_equal(security.check_equal_constant_time('', ''), True)
        assert_equal(security.check_equal_constant_time('abcabcabc', 'abcabcabc'), True)

    def test_check_equal_constant_time_returns_inequality(self):
        assert_equal(security.check_equal_constant_time(' ', ''), False)
        assert_equal(security.check_equal_constant_time('abcabcabc', 'abcabcabx'), False)
        assert_equal(security.check_equal_constant_time('abcabcabc', 'abcabcabx'), False)
        assert_equal(security.check_equal_constant_time('abcabcabc', 'abcabcabde'), False)
        assert_equal(security.check_equal_constant_time('abcabcabc', 'abcabcab'), False)

    def test_check_equal_constant_time_raises_an_error_on_non_strings(self):
        assert_raises(TypeError, security.check_equal_constant_time, (None, None))



import sys
import inspect
from operator import attrgetter
from functools import wraps, partial
from itertools import cycle

from fresco import GET, POST, PUT
from fresco.exceptions import BadRequest


def onerror(exceptions, handler):
    """\
    Return a decorator that can replace or update the return value of the
    function if an exception is raised
    """

    try:
        if isinstance(exceptions, Exception):
            exceptions = (exceptions,)
    except TypeError:
        pass

    def decorator(func):
        @wraps(func)
        def decorated(*fargs, **fkwargs):
            try:
                return func(*fargs, **fkwargs)
            except exceptions:
                exc_info = sys.exc_info()
                return handler(exc_info, *fargs, **fkwargs)
        return decorated
    return decorator


def extract_requestargs(requesttype, **argspec):
    """
    Decorator mapping request querystring/post data to function arguments.

    """
    request_attr = {
        GET: attrgetter('query'),
        POST: attrgetter('form'),
        PUT: attrgetter('form'),
    }[requesttype]

    strict_checking = argspec.pop('_strict', False)
    exception = argspec.pop('_raise_on_error', BadRequest)

    def decorator(func):
        """
        Decorate function ``func``.
        """

        f_args, f_varargs, f_varkw, f_defaults = inspect.getargspec(func)

        # Produce a mapping {argname: default}
        if f_defaults is None:
            f_defaults = []
        defaults = dict(zip(f_args[-len(f_defaults):], f_defaults))

        def decorated(*args, **kwargs):
            """
            Call ``func`` with arguments extracted from ``request``.
            """

            from fresco.core import context
            request = context.request
            given_arguments = dict(
                zip(f_args[:len(args)], args)
            )
            given_arguments.update(kwargs)
            newargs = given_arguments.copy()

            for name, type_fn in argspec.items():
                try:
                    try:
                        value = given_arguments[name]
                    except KeyError:
                        if isinstance(type_fn, list):
                            value = request_attr(request).getlist(name)
                        else:
                            value = request_attr(request)[name]

                    try:
                        if isinstance(type_fn, list):
                            value = [cast(v)
                                     for cast, v in zip(cycle(type_fn), value)]
                        else:
                            value = type_fn(value)
                    except ValueError as e:
                        if name in defaults and not strict_checking:
                            value = defaults[name]
                        else:
                            raise exception(
                                "Could not convert parameter %r to "\
                                " requested type (%s)" % (name, e.args[0])
                            )

                except KeyError:
                    try:
                        value = defaults[name]
                    except KeyError:
                        raise exception(name)
                newargs[name] = value

            return func(**newargs)
        return wraps(func)(decorated)
    return decorator

extract_getargs = partial(extract_requestargs, GET)
extract_postargs = partial(extract_requestargs, POST)
extract_getargs.__doc__ = """\
        Extract function arguments from request parameters. See extract_getargs
        for details.
        """
extract_getargs.__doc__ = """\
    Extract function arguments from request parameters

    Synopsis::

        >>> from flea import TestAgent
        >>> from fresco import FrescoApp, GET
        >>>
        >>> app = FrescoApp()
        >>> @app.route('/', GET)
        ... @extract_getargs(GET, id=int)
        ... def my_app(id):
        ...     return Response(['Got id %d' % (id,)])
        ...
        >>> print TestAgent(app).get('/?id=2')
        200 OK\r
        Content-Type: text/html; charset=UTF-8\r
        \r
        Got id 2

    If specified arguments are not present in the request (and no default value
    is given in the function signature), or a ValueError is thrown during type
    conversion an appropriate error will be raised::

        >>> print TestAgent(my_app.wsgi_app).get('/') #doctest: +ELLIPSIS
        Traceback (most recent call last):
            ...
        BadStatusError: GET '/' returned HTTP status '400 Bad Request'
        >>> print TestAgent(my_app.wsgi_app).get('/?id=ratatouille') #doctest: +ELLIPSIS
        Traceback (most recent call last):
            ...
        BadStatusError: GET '/' returned HTTP status '400 Bad Request'

    You can change the error raised, for example if you want a 404 page to be
    returned::

        >>> from fresco.exceptions import NotFound
        >>>
        >>> app = FrescoApp()
        >>> @app.route('/', GET)
        ... @extract_getargs(actions=[unicode])
        ... def my_app(id):
        ...     return Response(['Got id %d' % (id,)])
        ...
        >>> print TestAgent(app).get('/')
        Traceback (most recent call last):
            ...
        BadStatusError: GET '/' returned HTTP status '404 Not Found'

    A default argument value in the handler function will protect against this::

        >>> app = FrescoApp()
        >>> @app.route('/', GET)
        ... @extract_getargs(actions=[unicode])
        ... def my_app(id=1):
        ...     return Response(['Got id %d' % (id,)])
        ...
        >>> print TestAgent(app).get('/')
        200 OK\r
        Content-Type: text/html; charset=UTF-8\r
        \r
        Got id 1

    Sometimes it is necessary to map multiple request values to a single
    argument, for example in a form where two or more input fields have the
    same name. To do this, put the type-casting function into a list when
    calling ``with_request_args``::

        >>> app = FrescoApp()
        >>> @app.route('/', GET)
        ... @extract_getargs(actions=[unicode])
        ... def my_app(actions):
        ...     return Response([
        ...         ', '.join(actions)
        ...     ])
        ...
        >>> print TestAgent(app).get('/?actions=up;actions=up;actions=and+away%21')
        200 OK\r
        Content-Type: text/html; charset=UTF-8\r
        \r
        up, up, and away!

"""



from fresco.multidict import MultiDict
from nose.tools import assert_equal, assert_raises


class CheckedMultiDict(MultiDict):

    def check_consistency(self):
        "Verify that the MultiDict has internal consistency"
        dictallitems = sorted((k, v)
                              for k, vs in self._dict.items()
                              for v in vs)
        orderallitems = sorted(self._order)
        assert_equal(list(sorted(dictallitems)), list(sorted(orderallitems)))

MultiDict = CheckedMultiDict


class TestMultDict(object):

    def test_getitem_returns_only_first(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        assert_equal(m['a'], 1)

    def test_getitem_throws_keyerror(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        assert_raises(KeyError, lambda: m['c'])

    def test_get_returns_only_first(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        assert_equal(m.get('a'), 1)

    def test_get_returns_default(self):
        m = MultiDict()
        assert_equal(m.get('a'), None)
        assert_equal(m.get('a', 'foo'), 'foo')

    def test_getlist_returns_list(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert_equal(m.getlist('a'), [1, 2])
        assert_equal(m.getlist('b'), [])

    def test_copy_is_equal(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert_equal(list(m.allitems()),
                     list(m.copy().allitems()))

    def test_copy_is_independent(self):
        m = MultiDict([('a', 1), ('a', 2)])
        n = m.copy()
        n['b'] = 'foo'
        assert_equal(list(m.allitems()), [('a', 1), ('a', 2)])
        assert_equal(list(n.allitems()), [('a', 1), ('a', 2), ('b', 'foo')])

    def test_fromkeys(self):
        m = MultiDict.fromkeys(['a', 'b'])
        assert_equal(list(m.allitems()), [('a', None), ('b', None)])

    def test_fromkeys_with_value(self):
        m = MultiDict.fromkeys(['a', 'b'], 42)
        assert_equal(list(m.allitems()), [('a', 42), ('b', 42)])

    def test_items_only_returns_first_of_each(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert_equal(list(m.items()), [('a', 1)])

    def test_listitems_returns_lists(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert_equal(list(m.listitems()), [('a', [1, 2])])

    def test_allitems_returns_all(self):
        m = MultiDict([('a', 1), ('a', 2)])
        assert_equal(list(m.allitems()), [('a', 1), ('a', 2)])

    def test_allitems_returns_iterator(self):
        m = MultiDict([('a', 1), ('a', 2)])
        i = m.allitems()
        assert_equal(next(i), ('a', 1))
        assert_equal(next(i), ('a', 2))
        assert_raises(StopIteration, lambda: next(i))

    def test_keys(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        i = m.keys()
        assert_equal(next(i), 'a')
        assert_equal(next(i), 'b')
        assert_raises(StopIteration, lambda: next(i))

    def test_values(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        i = m.values()
        assert_equal(next(i), 1)
        assert_equal(next(i), 3)
        assert_raises(StopIteration, lambda: next(i))

    def test_listvalues(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        assert_equal(list(m.listvalues()), [[1, 2], [3]])

    def test_setitem(self):
        m = MultiDict()
        m['a'] = 1
        assert_equal(m['a'], 1)
        assert_equal(m.getlist('a'), [1])
        m.check_consistency()

    def test_setitem_replaces_existing(self):
        m = MultiDict()
        m['a'] = 1
        m['a'] = 2
        assert_equal(m.getlist('a'), [2])
        m.check_consistency()

    def test_delitem(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])

        del m['a']
        assert_equal(
            list(m.allitems()),
            [('b', 3)]
        )
        m.check_consistency()

        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        del m['b']
        assert_equal(
            list(m.allitems()),
            [('a', 1), ('a', 2)]
        )
        m.check_consistency()

    def test_iterable(self):
        m = MultiDict([('a', 1), ('a', 2), ('b', 3)])
        assert_equal(list(sorted(iter(m))), ['a', 'b'])

    def test_update_with_list(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.update([('a', 2), ('x', 2)])
        assert_equal(
            list(sorted(m.allitems())),
            [('a', 2), ('b', 2), ('x', 2)]
        )
        m.check_consistency()

    def test_update_with_dict(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.update({'a': 2, 'x': 2})
        assert_equal(
            list(sorted(m.allitems())),
            [('a', 2), ('b', 2), ('x', 2)]
        )
        m.check_consistency()

    def test_update_with_multidict(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.update(MultiDict([('a', 2), ('x', 2)]))
        assert_equal(
            list(sorted(m.allitems())),
            [('a', 2), ('b', 2), ('x', 2)]
        )
        m.check_consistency()

    def test_update_with_kwargs(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.update(a=2, x=2)
        assert_equal(
            sorted(list(m.allitems())),
            [('a', 2), ('b', 2), ('x', 2)]
        )
        m.check_consistency()

    def test_extend_with_list(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.extend([('a', 2), ('x', 2)])
        assert_equal(
            list(m.allitems()),
            [('a', 1), ('b', 2), ('a', 2), ('x', 2)]
        )
        m.check_consistency()

    def test_extend_with_dict(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.extend({'a': 2, 'x': 2})
        assert_equal(
            list(sorted(m.allitems())),
            [('a', 1), ('a', 2), ('b', 2), ('x', 2)]
        )
        m.check_consistency()

    def test_extend_with_multidict(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.extend(MultiDict([('a', 2), ('x', 2)]))
        assert_equal(
            list(sorted(m.allitems())),
            [('a', 1), ('a', 2), ('b', 2), ('x', 2)]
        )
        m.check_consistency()

    def test_extend_with_kwargs(self):
        m = MultiDict([('a', 1), ('b', 2)])
        m.extend(a=2, x=2)
        assert_equal(
            list(sorted(m.allitems())),
            [('a', 1), ('a', 2), ('b', 2), ('x', 2)]
        )
        m.check_consistency()

    def test_pop(self):
        m = MultiDict([('a', 1), ('b', 2), ('a', 3)])
        assert_equal(m.pop('b'), 2)
        assert_equal(list(sorted(m.allitems())), [('a', 1), ('a', 3)])
        m.check_consistency()

    def test_pop_returns_first_item(self):
        m = MultiDict([('a', 1), ('b', 2), ('a', 3)])
        assert_equal(m.pop('a'), 1)
        assert_equal(list(sorted(m.allitems())), [('a', 3), ('b', 2)])
        m.check_consistency()

    def test_pop_with_default(self):
        m = MultiDict([])
        assert_equal(m.pop('c', 'foo'), 'foo')
        m.check_consistency()

    def test_popitem(self):
        m = MultiDict([('a', 1), ('b', 2), ('b', 3)])

        assert_equal(m.popitem(), ('b', 2))
        m.check_consistency()
        assert_equal(m.popitem(), ('b', 3))
        m.check_consistency()
        assert_equal(m.popitem(), ('a', 1))
        m.check_consistency()

        try:
            m.popitem()
        except KeyError:
            pass
        else:
            assert False, "Expected KeyError in popitem from empty MultiDict"

    def test_len(self):
        m = MultiDict([('a', 1), ('b', 2), ('b', 3)])
        assert_equal(len(m), 2)




# encoding: utf-8
from functools import wraps
from mock import Mock
from nose.tools import assert_raises
from flea import TestAgent
from fresco import FrescoApp, GET, POST, Response
from fresco.exceptions import NotFound
from fresco.compat import ustring_type
from fresco.core import urlfor
from fresco.routing import Route, DelegateRoute, RouteCollection, routefor
from . import fixtures


class TestMethodDispatch(object):

    def test_route_is_dispatched_to_correct_method(self):

        getview = Mock(return_value=Response(content_type='text/plain'))
        postview = Mock(return_value=Response(content_type='text/plain'))
        app = FrescoApp()
        app.route('/', GET, getview)
        app.route('/', POST, postview)

        TestAgent(app).get('/')
        assert getview.call_count == 1
        assert postview.call_count == 0

        TestAgent(app).post('/')
        assert getview.call_count == 1
        assert postview.call_count == 1


class TestRouteDecorators(object):

    def exclaim(self, func):
        @wraps(func)
        def exclaim(*args, **kwargs):
            response = func(*args, **kwargs)
            return response.replace(content=[''.join(response.content) + '!'])
        return exclaim

    def test_decorator_is_applied(self):

        views = fixtures.CBV('test')

        app = FrescoApp()
        app.route('/decorated', GET, views.index_html,
                  decorators=[self.exclaim])
        app.route('/plain', GET, views.index_html)

        with app.requestcontext('/decorated'):
            assert app.view().content == ['test!']

        with app.requestcontext('/plain'):
            assert app.view().content == ['test']

    def test_decorator_works_with_urlfor(self):

        views = fixtures.CBV('test')
        app = FrescoApp()
        app.route('/decorated', GET, views.index_html,
                  decorators=[self.exclaim])
        with app.requestcontext():
            assert urlfor(views.index_html, _app=app) == \
                'http://localhost/decorated'

    def test_using_wraps_with_viewspec_doesnt_raise_AttributeError(self):

        def decorator(func):
            @wraps(func)
            def decorator(*args, **kwargs):
                return func(*args, **kwargs)
            return decorator

        class Views(object):
            __routes__ = Route('/',
                               GET, 'index_html', decorators=[decorator]),

            def index_html(self):
                return Response(['hello'])

        app = FrescoApp()
        app.include('/', Views())


class TestPredicates(object):

    def test_predicate_match(self):

        def v1():
            return Response(['x'])

        def v2():
            return Response(['y'])

        app = FrescoApp()
        app.route('/', GET, v1, predicate=lambda request: 'x' in request.query)
        app.route('/', GET, v2, predicate=lambda request: 'y' in request.query)

        t = TestAgent(app)
        assert t.get('/?x=1', check_status=False).body == 'x'
        assert t.get('/?y=1', check_status=False).body == 'y'
        assert t.get('/', check_status=False).response.status_code == 404


class TestRouteNames(object):

    def test_name_present_in_route_keys(self):
        r = Route('/', GET, None, name='foo')
        assert 'foo' in list(r.route_keys())

    def test_name_with_other_kwargs(self):
        r = Route('/', GET, None, name='foo', x='bar')
        assert 'foo' in list(r.route_keys())

    def test_name_cannot_contain_colon(self):
        assert_raises(ValueError, Route, '/', GET, None, name='foo:bar')


class TestRouteCollection(object):

    def test_get_routes_matches_on_method(self):

        r_get = Route('/', GET, None)
        r_post = Route('/', POST, None)

        rc = RouteCollection([r_post, r_get])

        assert [r.route for r in rc.get_routes('/', GET)] == [r_get]
        assert [r.route for r in rc.get_routes('/', POST)] == [r_post]

    def test_get_routes_matches_on_path(self):

        r1 = Route('/1', GET, None)
        r2 = Route('/2', GET, None)

        rc = RouteCollection([r1, r2])

        assert [r.route for r in rc.get_routes('/1', GET)] == [r1]
        assert [r.route for r in rc.get_routes('/2', GET)] == [r2]

    def test_get_routes_can_match_all_methods(self):

        r1 = Route('/1', GET, None)
        r2 = Route('/1', POST, None)

        rc = RouteCollection([r1, r2])
        assert [r.route for r in rc.get_routes('/1', None)] == [r1, r2]

    def test_route_returns_traversal_information_on_nested_routes(self):

        a = RouteCollection()
        b = RouteCollection()

        a.route('/harvey', GET, lambda: None)
        b.route('/harvey', GET, lambda: None)

        a.delegate('/rabbit', b)
        b.delegate('/hole', a)

        r = next(a.get_routes('/rabbit/hole/rabbit/harvey', None))
        assert r.collections_traversed == [(a, ''),
                                           (b, '/rabbit'),
                                           (a, '/rabbit/hole'),
                                           (b, '/rabbit/hole/rabbit')]


class TestRoutefor(object):

    def test_routefor_with_view_function(self):

        def view():
            return Response(['ok'])

        app = FrescoApp()
        route = app.route('/foo', GET, view)

        with app.requestcontext():
            assert routefor(view) == route

    def test_routefor_with_string(self):
        app = FrescoApp()
        route = app.route('/myviewfunc', GET, fixtures.module_level_function)
        with app.requestcontext():
            assert routefor('fresco.tests.fixtures.module_level_function') == \
                route

    def test_routefor_generates_first_route(self):

        myviewfunc = lambda req: Response([])
        app = FrescoApp()
        r1 = app.route('/1', GET, myviewfunc)
        app.route('/2', GET, myviewfunc)
        with app.requestcontext():
            assert routefor(myviewfunc) == r1


class TestDelegatedRoutes(object):

    def test_dispatch_to_delegated_route(self):

        def hello():
            return Response(['hello'])

        inner = FrescoApp()
        inner.route('/hello', GET, hello)

        outer = FrescoApp()
        outer.delegate('/say', inner)

        r = TestAgent(outer).get('/say/hello')
        assert r.body == 'hello'

    def test_url_variables_are_passed(self):

        hello = Mock()

        inner = FrescoApp()
        inner.route('/<i:str>', GET, hello)

        outer = FrescoApp()
        outer.delegate('/<o:str>', inner)

        with outer.requestcontext('/foo/bar'):
            outer.view()
            hello.assert_called_with(i='bar', o='foo')

    def test_delegation_to_dynamic_routes(self):

        result = []

        class MyRoutes(object):
            __routes__ = [Route('/<inner:int>/view', GET, 'view')]

            def __init__(self, **kwargs):
                self.kwargs = kwargs

            def view(self, **kwargs):
                result.append((self, kwargs))

        app = FrescoApp()
        app.delegate('/<outer:str>', MyRoutes, dynamic=True)
        with app.requestcontext('/two/2/view'):
            app.view()
            instance, inner_kwargs = result[0]
            assert inner_kwargs == {'inner': 2}
            assert instance.kwargs == {'outer': 'two'}

    def test_dynamic_routes_are_never_shared(self):

        result = []

        class MyRoutes(object):
            __routes__ = [Route('', GET, 'view')]

            def __init__(self, value):
                self.value = value

            def view(self):
                result.append(self.value)

        app = FrescoApp()
        app.delegate('/<value:str>', MyRoutes, dynamic=True)
        with app.requestcontext('/one'):
            app.view()
            v1 = result.pop()
        with app.requestcontext('/two'):
            app.view()
            v2 = result.pop()
        assert v1 == 'one'
        assert v2 == 'two', v2

    def test_pathfor_with_delegated_route(self):
        inner = FrescoApp()
        inner.route('/<i:str>', GET, lambda: None, name='inner-route')

        outer = FrescoApp()
        outer.delegate('/<o:str>', inner, name='delegation')

        with outer.requestcontext('/foo/bar'):
            assert outer.pathfor('delegation:inner-route',
                                 o='x', i='y') == '/x/y'

    def test_pathfor_with_dynamic_delegated_route(self):

        view = Mock()

        def routecollectionfactory(*args, **kwargs):
            return RouteCollection([Route('/<i:str>', GET, view,
                                          name='inner-route')])

        rc = RouteCollection()
        rc.delegate('/<o:str>', routecollectionfactory,
                    name='delegation', dynamic=True)

        assert rc.pathfor('delegation:inner-route', o='x', i='y') == '/x/y'

    def test_pathfor_with_dynamic_delegated_route_uses_default_args(self):

        view = Mock()

        def routecollectionfactory(factoryarg1, factoryarg2):
            return RouteCollection([Route('/<i:str>', GET, view,
                                          name='inner-route')])

        rc = RouteCollection()
        rc.delegate('/<factoryarg1:str>/<factoryarg2:str>',
                    routecollectionfactory,
                    factoryarg1_default='foo',
                    factoryarg2_default=lambda r: 'bar',
                    name='delegation', dynamic=True)

        assert rc.pathfor('delegation:inner-route', i='y') == '/foo/bar/y'

    def test_urlfor_with_dynamic_delegated_route_and_view_self(self):

        result = []

        class MyRoutes(object):
            __routes__ = [Route('/<inner:int>/view', GET, 'view')]

            def __init__(self, **kwargs):
                self.kwargs = kwargs

            def view(self, **kwargs):
                result.append(urlfor(self.view, inner=3))

        app = FrescoApp()
        app.delegate('/<outer:str>', MyRoutes, dynamic=True)
        with app.requestcontext('/two/2/view'):
            app.view()
            assert result == ['http://localhost/two/3/view']

    def test_urlgeneration_with_dynamic_routes(self):

        class Routable(object):
            __routes__ = [Route('/<b:int>', GET, 'view', name='y')]

            def __init__(self, a):
                pass

            def view(self, b):
                pass

        app = FrescoApp()
        app.delegate('/<a:str>', Routable, dynamic=True, name='x')
        with app.requestcontext('/two/2/view'):
            assert urlfor('x:y', a='a', b=1) == 'http://localhost/a/1'

    def test_delegated_routes_can_be_included(self):

        view = Mock()

        inner = RouteCollection([Route('/baz', GET, view)])
        middle = RouteCollection([DelegateRoute('/bar', inner)])
        outer = FrescoApp()
        outer.include('/foo', middle)
        with outer.requestcontext('/foo/bar/baz'):
            outer.view()
            view.assert_called()

    def test_not_found_is_returned(self):

        def inner():
            raise NotFound()

        outer = FrescoApp()
        outer.delegate('/foo', inner, dynamic=True)
        with outer.requestcontext('/foo/bar/baz'):
            response = outer.view()
            assert response.status_code == 404

    def test_not_found_causes_next_route_to_be_tried(self):

        def inner():
            raise NotFound()
        view = Mock()

        outer = FrescoApp()
        outer.delegate('/foo', inner, dynamic=True)
        outer.route('/foo', view)
        with outer.requestcontext('/foo'):
            outer.view()
            view.assert_called()


class TestConverters(object):

    def test_str_converter_returns_unicode(self):
        from fresco.routing import StrConverter
        s = ustring_type('abc')
        assert isinstance(StrConverter().from_string(s), ustring_type)



from fresco import Route, GET, Response


class CBV(object):
    """
    A class based view
    """

    __routes__ = [
        Route('/', GET, 'index_html'),
        Route('/page', GET, 'view_page'),
        Route('/page2', GET, 'view_page', tag='page2'),
    ]

    def __init__(self, s):
        self.s = s

    def index_html(self):
        return Response([self.s])

    def view_page(self):
        return Response([])


def module_level_function():
    """
    A module level function
    """



from operator import attrgetter, itemgetter, methodcaller
from fresco.exceptions import BadRequest
from itertools import cycle

__all__ = ('FormArg', 'GetArg', 'QueryArg', 'PostArg', 'CookieArg',
           'SessionArg', 'RequestObject', 'FormData')

_marker = []


class RouteKwarg(object):
    """
    RouteKwarg objects can be used as keyword arguments in a route definition.
    RouteKwargs can extract information from the request and make it available
    to the view.

    For example a RouteKwarg could be developed that reads information from the
    request cookie::

        Route('/', GET, affiliate=CookieArg())

    A naive implementation ``CookieArg`` could look like this::

        class CookieArg(RouteKwarg):

            def __call__(self, request):
                try:
                    return request.cookies[self.name].value
                except KeyError,
                    return None

    When the route is constructed the RouteKwarg's ``configure`` method will
    be called with the Route object and the keyword name.

    At every request, the RouteKwarg instance will be called and expected to
    supply the value for its argument.
    """

    route = None
    name = None

    def configure(self, route, name):
        self.route = route
        self.name = name

    def __call__(self, request):
        return None


class RequestArg(RouteKwarg):
    """\
    Extract a view keyword argument from the request object.
    """

    query = attrgetter('query')
    form = attrgetter('form')
    cookies = attrgetter('cookies')
    session = attrgetter('session')

    #: Source for the request variable
    source = form

    #: Exceptions that signal the converter could not do its job due to invalid
    #: input
    converter_exceptions = (ValueError, TypeError)

    def __init__(self, converter=None, key=None,
                 default=_marker, exception=BadRequest):

        self.formkey = key
        self.default = default
        self.required = default is _marker
        self.exception = exception
        self.is_list = isinstance(converter, list)
        if self.is_list:
            self.converter = lambda vs: [c(v)
                                         for c, v in zip(cycle(converter), vs)]
        else:
            self.converter = converter

    def configure(self, route, name):
        if self.formkey is None:
            self.formkey = name
        if self.is_list:
            self.getter = methodcaller('getlist', self.formkey)
        else:
            self.getter = itemgetter(self.formkey)

    def __call__(self, request):
        try:
            value = self.getter(self.source(request))
        except KeyError:
            if self.required:
                raise self.exception('No value provided for %s' %
                                     (self.formkey,))
            return self.default

        try:
            if self.converter is not None:
                value = self.converter(value)
        except self.converter_exceptions:
            raise self.exception('%r is not a valid value for %s' %
                                 (value, self.formkey,))

        return value


class RequestObject(RouteKwarg):
    """\
    Make the request object itself available as a view argument.

    Example::

        @app.route('/form', POST, request=RequestObject())
        def view(formdata):
            ...
    """
    def __call__(self, request):
        return request


class FormData(RouteKwarg):
    """\
    Make the request form data MultiDict available as a view argument.

    Example::

        @app.route('/form', POST, data=FormData())
        def view(formdata):
            ...
    """

    def __call__(self, request):
        return request.form


class CookieArg(RequestArg):
    """\
    Extract a view keyword argument from ``request.cookies``.

    Example::

        @app.route('/', GET, message=CookieArg(key='msg', default=None))
        def view(message):
            return Response([message])
    """
    source = RequestArg.cookies

    def getter(self, cookies):
        if self.is_list:
            return [c.value for c in cookies.getlist(self.formkey)]
        else:
            return cookies[self.formkey].value

    def configure(self, route, name):
        if self.formkey is None:
            self.formkey = name


class QueryArg(RequestArg):
    """\
    Extract a view keyword argument from ``request.query``.

    Example::

        @app.route('/add', GET, a=QueryArg(int), b=QueryArg(int))
        def view(a, b):
            return Response(['a + b = %d' % (a + b)])
    """
    source = RequestArg.query

#: Synonym for QueryArg
GetArg = QueryArg


class FormArg(RequestArg):
    """\
    Extract a view keyword argument from ``request.form``.

    Example::

        @app.route('/mul', POST, a=FormArg(int), b=FormArg(int))
        def view(a, b):
            return Response(['a * b = %d' % (a * b)])

    """
    source = RequestArg.form

#: Synonym for FormArg
PostArg = FormArg


class SessionArg(RequestArg):
    """\
    Extract a view keyword argument from ``request.form``.

    Example::

        @app.route('/mul', POST, a=FormArg(int), b=FormArg(int))
        def view(a, b):
            return Response(['a * b = %d' % (a * b)])

    """
    source = RequestArg.session



import sys

from collections import defaultdict
from contextlib import contextmanager
from functools import partial
from io import BytesIO

import blinker

from fresco.compat import unquote, urlparse
from fresco.request import Request
from fresco.response import Response
from fresco.util.urls import normpath
from fresco.util.wsgi import unicode_to_environ

from fresco.exceptions import ResponseException
from fresco.requestcontext import context
from fresco.routing import GET, ExtensiblePattern, RouteCollection, \
                           RouteKwarg, RouteNotFound
from fresco.options import Options

__all__ = ('FrescoApp', 'urlfor')


class FrescoApp(RouteCollection):
    """\
    Fresco application class.
    """

    #: The default class to use for URL pattern matching
    pattern_class = ExtensiblePattern

    #: A stdlib logger object, or None
    logger = None

    #: Class to use to instantiate request objects
    request_class = Request

    def __init__(self, views=None, path=None):

        super(FrescoApp, self).__init__()

        from fresco.middleware import context_middleware

        #: A list of (middleware, args, kwargs) tuples
        self._middleware = []

        #: Middleware layers applied after any middleware added through
        #: :meth:`add_middleware`.
        self.core_middleware = [partial(context_middleware, frescoapp=self)]

        #: The WSGI application. Replaced every time new middleware is added.
        self._wsgi_app = self._make_wsgi_app()

        #: An options dictionary, for arbitrary application variables or
        #: configuration
        self.options = Options()

        self.signal_ns = blinker.Namespace()

        #: Sent when a request matches a route, immediately before the view is
        #: invoked.
        #: Receivers will be passed: route, view, view_args, view_kwargs
        self.route_matched = self.signal_ns.signal('route_matched')

        #: Sent Sent after a view function is invoked, before the response
        #: object is output.
        #: Receivers will be passed: view, request and response
        self.view_finished = self.signal_ns.signal('view_finished')

        if views:
            if path is None:
                path = '/'
            self.include(path, views)

    def __call__(self, environ, start_response):
        """\
        Call the app as a WSGI application
        """
        return self._wsgi_app(environ, start_response)

    def __str__(self):
        """\
        String representation of the application and its configured routes
        """
        s = '<%s\n' % self.__class__.__name__
        for route in self.__routes__:
            s += '    ' + str(route)
        s += '>'
        return s

    def view(self, routecollection=None, request=None, path=None, method=None):
        routecollection = routecollection or self
        request = request or context.request
        method = method or request.method
        path = path or request.path_info
        error_response = None

        context.app = self
        environ = request.environ
        environ['fresco.app'] = self

        if path:
            path = normpath(path)
        else:
            path = '/'

        for traversal in routecollection.get_routes(path, method):

            route = traversal.route
            try:
                if route.predicate and not route.predicate(request):
                    continue

                for k, v in traversal.kwargs.items():
                    if isinstance(v, RouteKwarg):
                        v = v(request)
                    traversal.kwargs[k] = v

                environ['wsgiorg.routing_args'] = (traversal.args,
                                                   traversal.kwargs)
                view = route.getview()
                context.view_self = getattr(view, '__self__', None)

                if self.logger:
                    self.logger.info("matched route: %s %r => %r",
                                     request.method, path, view)

                context.route_traversal = traversal
                self.route_matched.send(self, route=route, view=view,
                                        request=request)

                response = route.getdecoratedview()(*traversal.args,
                                                    **traversal.kwargs)
            except ResponseException as e:
                if e.is_final:
                    return e.response
                error_response = error_response or e.response
            else:
                self.view_finished.send(self, view=view, response=response)
                return response

        # A route was matched, but an error was returned
        if error_response:
            return error_response

        # Is this a head request?
        if method == 'HEAD':
            response = self.view(routecollection, request, path, GET)
            if 200 <= response.status_code < 300:
                return response.replace(content=[], content_length=0)
            return response

        # Is the URL matched by another HTTP method?
        methods = self.get_methods(routecollection, request, path)
        if methods:
            return Response.method_not_allowed(methods)

        # Is the URL just missing a trailing '/'?
        if path[-1] != '/':
            for _ in self.get_methods(routecollection, request, path + '/'):
                return Response.redirect_permanent(path + '/')

        return Response.not_found()

    def get_methods(self, routecollection, request, path):
        """\
        Return the HTTP methods valid in routes to the given path
        """
        methods = set()
        for route, _, _, _ in routecollection.get_routes(path, None):
            if route.predicate and not route.predicate(request):
                continue
            methods.update(route.methods)
        return methods

    def add_middleware(self, middleware, *args, **kwargs):
        """\
        Add a WSGI middleware layer
        """
        self._middleware.append((middleware, args, kwargs))
        self._wsgi_app = self._make_wsgi_app()

    def _raw_wsgi_app(self, environ, start_response):
        return self.view(self, self.request_class(environ))(environ,
                                                            start_response)

    def _make_wsgi_app(self, app=None):
        app = app or self._raw_wsgi_app
        for middleware, args, kwargs in self._middleware:
            app = middleware(app, *args, **kwargs)
        for middleware in self.core_middleware:
            app = middleware(app)
        return app

    def urlfor(self, viewspec, *args, **kwargs):
        """\
        Return the url for the given view name or function spec

        :param viewspec: a view name, a reference in the form
                         ``'package.module.viewfunction'``, or the view
                         callable itself.
        :param _scheme: the URL scheme to use (eg 'https' or 'http').
        :param _netloc: the network location to use (eg 'localhost:8000').
        :param _script_name: the SCRIPT_NAME path component
        :param _query: any query parameters, as a dict or list of
                        ``(key, value)`` tuples.
        :param _fragment: a URL fragment to append.

        All other arguments or keyword args are fed to the ``pathfor`` method
        of the pattern.
        """
        scheme = kwargs.pop('_scheme', None)
        netloc = kwargs.pop('_netloc', None)
        query = kwargs.pop('_query', {})
        script_name = kwargs.pop('_script_name', None)
        fragment = kwargs.pop('_fragment', None)

        request = context.request
        traversal = context.get('route_traversal')
        if traversal and traversal.collections_traversed:
            collections_traversed = traversal.collections_traversed
        else:
            collections_traversed = [(self, '')]

        exc = None
        for collection, base_path in collections_traversed[::-1]:
            try:
                path = base_path + \
                        collection.pathfor(viewspec, request=request,
                                           *args, **kwargs)
            except RouteNotFound as e:
                exc = e
                continue
            return request.make_url(scheme=scheme, netloc=netloc,
                                    SCRIPT_NAME=script_name,
                                    PATH_INFO=path,
                                    parameters='',
                                    query=query, fragment=fragment)
        raise exc or RouteNotFound(viewspec)

    def view_method_not_found(self, valid_methods):
        """
        Return a ``405 Method Not Allowed`` response.

        Called when a view matched the pattern but no HTTP methods matched.
        """
        return Response.method_not_allowed(valid_methods)

    @contextmanager
    def requestcontext(self, url='http://localhost/', environ=None,
                       wsgi_input=b'', **kwargs):
        """
        Return the global :class:`fresco.requestcontext.RequestContext`
        instance, populated with a new request object modelling default
        WSGI environ values.

        Synopsis::

            >>> app = FrescoApp()
            >>> with app.requestcontext('http://www.example.com/view') as c:
            ...     print c.request.url
            ...
            http://www.example.com/view

        Note that ``url`` must be properly URL encoded.
        """
        url = urlparse(url)
        netloc = url.netloc
        user = ''
        if '@' in netloc:
            user, netloc = netloc.split('@', 1)

        if ':' in user:
            user, _ = user.split(':')[0]

        if isinstance(wsgi_input, bytes):
            wsgi_input = BytesIO(wsgi_input)

        env_overrides = environ or {}
        for key, value in kwargs.items():
            key = key.replace('wsgi_', 'wsgi.')
            if '.' not in key:
                value = unicode_to_environ(value)
            env_overrides[key] = value

        environ = {
            'REQUEST_METHOD': 'GET',
            'SERVER_NAME': 'localhost',
            'SERVER_PORT': '80',
            'SERVER_PROTOCOL': 'HTTP/1.0',
            'HTTP_HOST': netloc or 'localhost',
            'SCRIPT_NAME': '',
            'PATH_INFO': unicode_to_environ(unquote(url.path)),
            'REMOTE_ADDR': '127.0.0.1',
            'wsgi.version': (1, 0),
            'wsgi.url_scheme': url.scheme or 'http',
            'wsgi.input': wsgi_input,
            'wsgi.errors': sys.stderr,
            'wsgi.multithread': True,
            'wsgi.multiprocess': True,
            'wsgi.run_once': False,
        }
        if url.scheme == 'https':
            environ['HTTPS'] = 'on'
            environ['SERVER_PORT'] = '443'

        if user:
            environ['REMOTE_USER'] = user

        if url.query:
            environ['QUERY_STRING'] = url.query

        environ.update(env_overrides)
        context.push(request=self.request_class(environ), app=self)

        def _fakewsgiapp(environ, start_response):
            start_response('200 OK', [])
            yield b''
        _fakewsgiapp = self._make_wsgi_app(_fakewsgiapp)

        # Invoke a fake wsgi app wrapped in all middleware layers. This
        # executes any middleware, which may have side effects which are
        # expected downstream (eg setting environ keys)
        wsgi_iter = _fakewsgiapp(environ, lambda *a, **kw: None)
        next(wsgi_iter)
        yield context

        # Exhaust the iterator and close up
        list(wsgi_iter)
        getattr(wsgi_iter, 'close', lambda: None)()
        context.pop()


def collect_keys(items):
    """
    [(k1, v1), (k1, v2), (k2, v3)] -> [(k1, [v1, v2]), (k2, [v3])]
    """
    d = defaultdict(list)
    for key, value in items:
        d[key].append(value)
    return d.items()


def urlfor(*args, **kwargs):
    """\
    Convenience wrapper around :meth:`~fresco.core.FrescoApp.urlfor`.
    """
    return context.app.urlfor(*args, **kwargs)
